package com.clx.performance.service.impl;

import com.clx.open.sdk.request.action.*;
import com.clx.performance.constant.RedisConstants;
import com.clx.performance.dao.OrderChildExceptionReportDao;
import com.clx.performance.enums.ExceptionReportEnum;
import com.clx.performance.enums.SyncPlatformEnum;
import com.clx.performance.model.OrderChildExceptionReport;
import com.clx.performance.service.OrderChildExceptionReportService;
import com.clx.performance.struct.OrderChildExceptionReportStruct;
import com.clx.performance.utils.LocalDateTimeUtils;
import com.msl.common.result.Result;
import com.msl.common.utils.DateUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author kavin
 * Date 2024-10-22
 * Time 10:38
 */
@Service
@Slf4j
@AllArgsConstructor
public class OrderChildExceptionReportServiceImpl  implements OrderChildExceptionReportService {

    private final OrderChildExceptionReportStruct orderChildExceptionReportStruct;
    private final OrderChildExceptionReportDao orderChildExceptionReportDao;
    private final RedisTemplate<String,Integer> redisTemplate;

    @Override
    public Result<String> smbSyncExceptionReportInfo(SmbSyncExceptionReportInfoAction action) {
        OrderChildExceptionReport report = orderChildExceptionReportStruct.convertInfoSmbAction(action);
        return Result.ok(saveExceptionReportInfo(report, SyncPlatformEnum.Source.TRADE_PLATFORM.getCode()));
    }

    @Override
    public Result<Object> smbSyncExceptionReportResult(SmbSyncExceptionReportResultAction action) {
        return null;
    }

    @Override
    public Result<Object> smbWithdrawExceptionReport(SmbWithdrawExceptionReportAction action) {
        return null;
    }

    @Override
    public Result<String> mslSyncExceptionReportInfo(MslSyncExceptionReportInfoAction action) {
        OrderChildExceptionReport report = orderChildExceptionReportStruct.convertInfoMslAction(action);
        return Result.ok(saveExceptionReportInfo(report, SyncPlatformEnum.Source.NEW_OWNER_CLIENT.getCode()));
    }

    @Override
    public Result<Object> mslSyncExceptionReportResult(MslSyncExceptionReportResultAction action) {
        return null;
    }

    @Override
    public Result<Object> mslWithdrawExceptionReport(MslWithdrawExceptionReportAction action) {
        return null;
    }


    private String saveExceptionReportInfo(OrderChildExceptionReport report,Integer source){
        report.setReportNo(getReportNo());
        report.setSource(source);
        report.setStatus(ExceptionReportEnum.Status.WAIT_DEAL.getCode());
        orderChildExceptionReportDao.saveEntity(report);
        return report.getReportNo();
    }


    public String getReportNo(){
        LocalDateTime begin = LocalDateTime.now();
        String datetime = DateUtils.formatDateTime(begin,"yyyyMMdd").get();
        String reportCacheKey = getReportCacheKey(datetime);
        int randomNumber = (int) (Math.random() * 99999999) + 10000000;

        boolean isFirst = false;

        if (Boolean.TRUE.equals(redisTemplate.hasKey(reportCacheKey))){
            List<Integer> range = redisTemplate.opsForList().range(reportCacheKey, 0, -1);
            if(CollectionUtils.isNotEmpty(range)){
                //如果缓存中包含生成的数字，则重新获取
                while (range.contains(randomNumber)){
                    randomNumber = (int) (Math.random() * 99999999) + 10000000;
                }
                redisTemplate.opsForList().leftPush(reportCacheKey,randomNumber);
            }else{
                isFirst = true;
            }
        }else{   //首次
            isFirst = true;
        }
        if(isFirst){
            redisTemplate.opsForList().leftPush(reportCacheKey,randomNumber);
            LocalDateTime end = LocalDateTimeUtils.getDayEnd(begin);
            long seconds = LocalDateTimeUtils.betweenSecond(begin, end);
            redisTemplate.expire(reportCacheKey,seconds, TimeUnit.SECONDS);
        }
        return "YCSB" + datetime + randomNumber;

    }
    public String getReportCacheKey(String datetime){
        return RedisConstants.EXCEPTION_REPORT_CACHE_KEY + datetime;
    }

    public static void main(String[] args) {
        LocalDateTime begin = LocalDateTime.now();

        LocalDateTime end = LocalDateTimeUtils.getDayEnd(begin);

        long seconds = LocalDateTimeUtils.betweenSecond(begin, end);

        System.out.println(seconds);


    }
}
